Class: Excon::ResolverFactory
- Inherits:
-
Object
- Object
- Excon::ResolverFactory
- Defined in:
- lib/excon/resolver_factory.rb
Overview
This factory produces new resolv gem resolver instances. Users who wants to configure a custom resolver (varying settings for varying resolvers) can provide a custom resolver factory class and configure it globally on the Excon defaults:
Excon.defaults[:resolver_factory] = MyCustomResolverFactory
Then you just need to provide a static method called .create_resolver which returns a new Resolv instance. This allows the customization.
Class Method Summary collapse
-
.create_resolver ⇒ Resolv
The new resolver instance.
Class Method Details
.create_resolver ⇒ Resolv
Returns the new resolver instance.
14 15 16 |
# File 'lib/excon/resolver_factory.rb', line 14 def self.create_resolver Resolv.new end |